$ifndef _DAC_INC_
$define _DAC_INC_
'
' ------------------------------------------------------------------------------------------------------------
'| This source code is provided 'as is' and any express or implied warranties, including,                     |
'| but not limited to, the implied warranties of merchantability and fitness for a particular                 |
'| purpose are disclaimed.                                                                                    |
'|                                                                                                            |
'| In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,    |
'| special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute    |
'| goods or services; loss of use, data, or profits;  or business interruption) however caused and on any     |
'| theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)    |
'| arising in any way out of the use of this source code, even if advised of the possibility of such damage.  |
'|                                                                                                            |
'| Any views and conclusions contained in the source code and any documentation are those of the author's     |
'| and should not be interpreted as representing official policies,                                           |
'| either expressed or implied, of Crownhill Associates Ltd.                                                  |
' ------------------------------------------------------------------------------------------------------------
'
' 16-bit DAC peripheral procedures for Proton24
' Written by Les Johnson 10-05-14
'
' Constants used to setup the DAC as signed or unsigned
'
$ifndef cDAC_Unsigned
    $define cDAC_Unsigned 0
$endif
$ifndef cDAC_Signed
    $define cDAC_Signed 1
$endif
'--------------------------------------------------------------------------------
' Initialise the Left and Right DACs
' Input     : pClockDiv holds the high speed clock divisor value (see the datasheet for details)
'           : pSigned = 1 if DACs are signed, 0 if unsigned
' Output    : None
' Notes     : None
'
Proc DAC1_Init(pClockDiv As Byte, pSigned As Bit)
'
' ACLKCON setup
'
    ACLKCONbits_SELACLK = 0                     ' PLL output (Fosc) provides the source clock for the Auxiliary Clock Divider
    ACLKCONbits_AOSCMD1 = 0                     ' \ Auxiliary Oscillator Disabled
    ACLKCONbits_AOSCMD0 = 0                     ' /
    ACLKCONbits_APSTSCLR2 = 1                   ' \
    ACLKCONbits_APSTSCLR1 = 1                   ' |  Auxiliary Clock Output divided by 1
    ACLKCONbits_APSTSCLR0 = 1                   ' /
    ACLKCONbits_ASRCSEL = 0                     ' Auxiliary Oscillator is the Clock Source
'
' DAC1 setup
'
    DAC1DFLT = $0000                            ' DAC default value
    Dec pClockDiv                               ' \
    DAC1CON.Byte0 = DAC1CON.Byte0 & %11110000   ' | Divide High Speed Clock
    DAC1CON.Byte0 = DAC1CON.Byte0 | pClockDiv   ' /

    DAC1CONbits_FORM = pSigned                  ' Data Format: signed or unsigned integer
    DAC1CONbits_AMPON = 0                       ' Analogue Output Amplifier is disabled during Sleep Mode/Stop-in Idle mode. All channels are reset

    IFS4bits_DAC1LIF = 0                        ' Clear left Channel Interrupt Flag
    IEC4bits_DAC1LIE = 0                        ' Left Channel Interrupt disabled
    IFS4bits_DAC1RIF = 0                        ' Clear Right Channel Interrupt Flag
    IEC4bits_DAC1RIE = 0                        ' Right Channel Interrupt disabled
    DAC1CONbits_DACEN = 1                       ' DAC1 Module Enabled
EndProc

'--------------------------------------------------------------------------------
' Enable the Left Channel DAC
' Input     : pInterrupt = 1 For Interrupt enabled
' Output    : None
' Notes     : None
'
Proc DAC1L_Enable(pInterrupt As Bit)
    DAC1STATbits_LOEN = 1           ' Left Channel DAC Output Enabled
    DAC1STATbits_LITYPE = 1         ' Left Channel Interrupt if FIFO is empty
    DAC1STATbits_LMVOEN = 0         ' Left Channel Midpoint output is disabled

    IFS4bits_DAC1LIF = 0            ' Clear left Channel Interrupt Flag
    IEC4bits_DAC1LIE = pInterrupt   ' Left Channel Interrupt Enable
EndProc

'--------------------------------------------------------------------------------
' Enable the Right Channel DAC
' Input     : pInterrupt = 1 for interrupt enabled
' Output    : None
' Notes     : None
'
Proc DAC1R_Enable(pInterrupt As Bit)
    DAC1STATbits_ROEN = 1           ' Right Channel DAC Output Enabled
    DAC1STATbits_RITYPE = 1         ' Right Channel Interrupt if FIFO is Empty
    DAC1STATbits_LMVOEN = 0         ' Right Channel Midpoint output is disabled

    IFS4bits_DAC1RIF = 0            ' Clear Right Channel Interrupt Flag
    IEC4bits_DAC1RIE = pInterrupt   ' Right Channel Interrupt Enable
EndProc

'--------------------------------------------------------------------------------
' Write to the Left Channel DAC
' Input     : pValue holds the 16-bit value to write to the DAC
' Output    : None
' Notes     : None
'
$define DAC1L_Write(pValue) DAC1LDAT = pValue

'--------------------------------------------------------------------------------
' Write to the Right Channel DAC
' Input     : pValue holds the 16-bit value to write to the DAC
' Output    : None
' Notes     : None
'
$define DAC1R_Write(pValue) DAC1RDAT = pValue

$endif  ' _DAC_INC_

