'
' ------------------------------------------------------------------------------------------------------------
'| This source code is provided 'as is' and any express or implied warranties, including,                     |
'| but not limited to, the implied warranties of merchantability and fitness for a particular                 |
'| purpose are disclaimed.                                                                                    |
'|                                                                                                            |
'| In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,    |
'| special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute    |
'| goods or services; loss of use, data, or profits;  or business interruption) however caused and on any     |
'| theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)    |
'| arising in any way out of the use of this source code, even if advised of the possibility of such damage.  |
'|                                                                                                            |
'| Any views and conclusions contained in the source code and any documentation are those of the author's     |
'| and should not be interpreted as representing official policies,                                           |
'| either expressed or implied, of Crownhill Associates Ltd.                                                  |
' ------------------------------------------------------------------------------------------------------------
'
' 100Hz FIR Highpass Filter Example for Proton24
' Written by Les Johnson 10-05-14
'
' Place a signal on AN0 and the filtered signal appears on the left channel DAC
' An oscilloscope placed on the DAC will show the signal alter when the desired filter frequency/s are reached
'
' The signal on AN0 should be no more than 1.0 Volt and have a DC offset of 1.7 Volts.
'
    Device = 33FJ128GP802                                                   ' Choose a dsPIC33F device with a DAC peripheral
    Declare Xtal = 79.23                                                    ' Operate the device at 79.23MHz

    Include "DAC.inc"                                                       ' Load the DAC procedures into the program
    Include "FIR.inc"                                                       ' Load the FIR procedures into the program
'
' Timer1 interrupt (ADC sample) rate calculator
'
    $define cSample_Rate 44100                                              ' Sample rate of the ADC (in Hz)
    $define cMicroSeconds $eval (1000000 / cSample_Rate)                    ' Interrupt rate (in us). Sample rate is (1000000 / Frequency (in Hz))
    $define cTweak_Value 36                                                 ' Added tweak value for rounding errors
    $define cPR1_Value $eval (cMicroSeconds * (_xtal / 2)) + cTweak_Value   ' Calculate the actual value to place into the PR1 SFR
'
' Do some error checking for the PR1 value
'
$if cPR1_Value > 65535
    $error "cPR1_Value too large for interrupt duration"
$elseif cPR1_Value = 0
    $error "cPR1_Value too small for interrupt duration"
$endif

    $define cBUFFER_SIZE 512                                                ' The size of the YRAM buffer
    $define cFILTER_TAPS 512                                                ' The amount of filter taps (the coefficient table)
'
' Coefficient table addressable in PSV space
' The values below are for a 100Hz high pass filter using a Hamming window
'
    Dim cCOEFF As PSV = As Word $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003,_
                                $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003,_
                                $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003,_
                                $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003,_
                                $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0002, $0002,_
                                $0002, $0002, $0002, $0002, $0002, $0001, $0001, $0001, $0001, $0001, $0001, $0000,_
                                $0000, $0000, $0000, $FFFF, $FFFF, $FFFF, $FFFE, $FFFE, $FFFE, $FFFD, $FFFD, $FFFD,_
                                $FFFC, $FFFC, $FFFC, $FFFB, $FFFB, $FFFA, $FFFA, $FFF9, $FFF9, $FFF8, $FFF8, $FFF7,_
                                $FFF7, $FFF6, $FFF5, $FFF5, $FFF4, $FFF4, $FFF3, $FFF2, $FFF2, $FFF1, $FFF0, $FFEF,_
                                $FFEF, $FFEE, $FFED, $FFEC, $FFEC, $FFEB, $FFEA, $FFE9, $FFE8, $FFE7, $FFE6, $FFE5,_
                                $FFE4, $FFE3, $FFE2, $FFE1, $FFE0, $FFDF, $FFDE, $FFDD, $FFDC, $FFDB, $FFDA, $FFD9,_
                                $FFD8, $FFD6, $FFD5, $FFD4, $FFD3, $FFD2, $FFD0, $FFCF, $FFCE, $FFCD, $FFCB, $FFCA,_
                                $FFC9, $FFC7, $FFC6, $FFC5, $FFC3, $FFC2, $FFC1, $FFBF, $FFBE, $FFBC, $FFBB, $FFBA,_
                                $FFB8, $FFB7, $FFB5, $FFB4, $FFB2, $FFB1, $FFAF, $FFAE, $FFAC, $FFAB, $FFA9, $FFA8,_
                                $FFA6, $FFA5, $FFA3, $FFA2, $FFA0, $FF9F, $FF9D, $FF9C, $FF9A, $FF99, $FF97, $FF96,_
                                $FF94, $FF93, $FF91, $FF90, $FF8E, $FF8D, $FF8B, $FF8A, $FF88, $FF87, $FF86, $FF84,_
                                $FF83, $FF81, $FF80, $FF7E, $FF7D, $FF7C, $FF7A, $FF79, $FF78, $FF76, $FF75, $FF74,_
                                $FF72, $FF71, $FF70, $FF6F, $FF6D, $FF6C, $FF6B, $FF6A, $FF69, $FF67, $FF66, $FF65,_
                                $FF64, $FF63, $FF62, $FF61, $FF60, $FF5F, $FF5E, $FF5D, $FF5C, $FF5B, $FF5B, $FF5A,_
                                $FF59, $FF58, $FF57, $FF57, $FF56, $FF55, $FF55, $FF54, $FF54, $FF53, $FF52, $FF52,_
                                $FF51, $FF51, $FF51, $FF50, $FF50, $FF4F, $FF4F, $FF4F, $FF4F, $FF4E, $FF4E, $FF4E,_
                                $FF4E, $FF4E, $FF4E, $FF4E, $7F4E, $FF4E, $FF4E, $FF4E, $FF4E, $FF4E, $FF4E, $FF4E,_
                                $FF4F, $FF4F, $FF4F, $FF4F, $FF50, $FF50, $FF51, $FF51, $FF51, $FF52, $FF52, $FF53,_
                                $FF54, $FF54, $FF55, $FF55, $FF56, $FF57, $FF57, $FF58, $FF59, $FF5A, $FF5B, $FF5B,_
                                $FF5C, $FF5D, $FF5E, $FF5F, $FF60, $FF61, $FF62, $FF63, $FF64, $FF65, $FF66, $FF67,_
                                $FF69, $FF6A, $FF6B, $FF6C, $FF6D, $FF6F, $FF70, $FF71, $FF72, $FF74, $FF75, $FF76,_
                                $FF78, $FF79, $FF7A, $FF7C, $FF7D, $FF7E, $FF80, $FF81, $FF83, $FF84, $FF86, $FF87,_
                                $FF88, $FF8A, $FF8B, $FF8D, $FF8E, $FF90, $FF91, $FF93, $FF94, $FF96, $FF97, $FF99,_
                                $FF9A, $FF9C, $FF9D, $FF9F, $FFA0, $FFA2, $FFA3, $FFA5, $FFA6, $FFA8, $FFA9, $FFAB,_
                                $FFAC, $FFAE, $FFAF, $FFB1, $FFB2, $FFB4, $FFB5, $FFB7, $FFB8, $FFBA, $FFBB, $FFBC,_
                                $FFBE, $FFBF, $FFC1, $FFC2, $FFC3, $FFC5, $FFC6, $FFC7, $FFC9, $FFCA, $FFCB, $FFCD,_
                                $FFCE, $FFCF, $FFD0, $FFD2, $FFD3, $FFD4, $FFD5, $FFD6, $FFD8, $FFD9, $FFDA, $FFDB,_
                                $FFDC, $FFDD, $FFDE, $FFDF, $FFE0, $FFE1, $FFE2, $FFE3, $FFE4, $FFE5, $FFE6, $FFE7,_
                                $FFE8, $FFE9, $FFEA, $FFEB, $FFEC, $FFEC, $FFED, $FFEE, $FFEF, $FFEF, $FFF0, $FFF1,_
                                $FFF2, $FFF2, $FFF3, $FFF4, $FFF4, $FFF5, $FFF5, $FFF6, $FFF7, $FFF7, $FFF8, $FFF8,_
                                $FFF9, $FFF9, $FFFA, $FFFA, $FFFB, $FFFB, $FFFC, $FFFC, $FFFC, $FFFD, $FFFD, $FFFD,_
                                $FFFE, $FFFE, $FFFE, $FFFF, $FFFF, $FFFF, $0000, $0000, $0000, $0000, $0001, $0001,_
                                $0001, $0001, $0001, $0001, $0002, $0002, $0002, $0002, $0002, $0002, $0002, $0003,_
                                $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003,_
                                $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003,_
                                $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003,_
                                $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003,_
                                $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003, $0003
'
' Variables used for the filter
'
    Dim Global_wFilterValue As SWord                    ' Final value from the filter
    Dim Global_wInIndex As Word                         ' Input buffer index
    Dim Global_wInputBuf[cBUFFER_SIZE] As Word YRAM     ' Input buffer, must be in Y RAM

'----------------------------------------------------------------------------------------------------
' Setup the ADC and initialise an ADC interrupt
' Input     : None
' Output    : None
' Notes     : The procedure also encompasses the interrupt handler
'
Proc ADC_Start()
    Input PORTA.0                           ' Make AN0 an input
    AD1CON2 = 0                             ' AVdd, AVss, MUXA only
    AD1PCFGLbits_PCFG0 = 0                  ' Analogue input on AN0
    AD1CHS0 = 0                             ' Configure AN0 as the ADC input
    AD1CON2 = $0000
    AD1CON3 = $021A                         ' Sampling time = 3 * Tad, minimum Tad selected
    AD1CON1 = %1000001111100010
'              ||||||||||||||||______________ Bit-0 : No ADC conversion yet
'              |||||||||||||||_______________ Bit-1 : ADC Sample and Hold amplifiers are holding
'              ||||||||||||||________________ Bit-2 : Sampling begins immediately after last conversion. SAMP bit is auto-set.
'              |||||||||||||_________________ Bit-3 : Samples multiple channels individually in sequence
'              ||||||||||||__________________ Bit-4 : Sample Clock Source Group Bit
'              |||||||||||___________________ Bit-5 :\
'              ||||||||||____________________ Bit-6 :| Internal Counter ends sampling and starts conversion (auto-convert)
'              |||||||||_____________________ Bit-7 :/
'              ||||||||______________________ Bit-8 :\ ADC is using signed fractional (DOUT = sddddddddddd)
'              |||||||_______________________ Bit-9 :/
'              ||||||________________________ Bit-10: 12-Bit, 1-channel ADC operation
'              |||||_________________________ Bit-11: Unimplemented
'              ||||__________________________ Bit-12: DMA buffers are written in Scatter/Gather mode
'              |||___________________________ Bit-13: Continue module operation in Idle mode
'              ||____________________________ Bit-14: Unimplemented
'              |_____________________________ Bit-15: Enable the ADC

    IEC0bits_AD1IE = 1                      ' ADC interrupt enabled
    AD1CON1bits_ADON = 1                    ' Enable the ADC
    AD1CON1bits_SAMP = 1                    ' Start a sample
'
'-------------------------------------------
' ADC interrupt handler
' Input     : None
' Output    : Global_wFilterValue holds the filtered value
' Notes     : Filters the ADC result
'
Isr ADC1Interrupt
    Global_wInputBuf[Global_wInIndex] = ADCBUF0             ' Get the ADC sample
'
' Filter the ADC samples into Global_wFilterValue
'
    Global_wFilterValue = FIR_Filter(FIRStruct, Global_wInIndex)
'
' Manage the filter's buffer
'
    Inc Global_wInIndex                                     ' Increment the samples buffer index
    If Global_wInIndex >= cBUFFER_SIZE Then                 ' Have we reached the end of the buffer?
        Global_wInIndex = 0                                 ' Yes. So reset it
    EndIf
    IFS0bits_AD1IF = 0                                      ' Clear the ADC interrupt flag
EndIsr
EndProc

'----------------------------------------------------------------------------------------------------
' Initialise a Timer1 interrupt
' Input     : None
' Output    : None
' Notes     : The procedure also encompasses the Interrupt handler
'
Proc Timer1_Start()
    T1CON = 0                                   ' Reset the T1CON SFR
    T1CONbits_TCKPS0 = 0                        ' \
    T1CONbits_TCKPS1 = 0                        ' / Timer1 Prescaler 1:1
    PR1 = cPR1_Value                            ' ADC Sampling rate
                                                ' Set the match cPR1_Value (will cause an interrupt when this value is reached)
    TMR1 = 0                                    ' Reset Timer1 value
    IFS0bits_T1IF = 0                           ' Clear the Timer1 interrupt flag
    IPC0bits_T1IP0 = 1                          ' \
    IPC0bits_T1IP1 = 1                          ' | High Priority Timer1 interrupt
    IPC0bits_T1IP2 = 1                          ' /
    T1CONbits_TON = 1                           ' Turn on Timer1
    IEC0bits_T1IE = 1                           ' Enable the Timer1 interrupt
'
'-----------------------------------------------
' Timer1 Interrupt handler
' Input     : None
' Output    : None
' Notes     : Triggers an ADC interrupt
'
Isr T1Interrupt
    If AD1CON1bits_DONE = 1 Then                ' If ADC is not busy
        AD1CON1bits_SAMP = 1                    ' Start new sample
    EndIf
    IFS0bits_T1IF = 0                           ' Clear the Timer1 interrupt flag
EndIsr
EndProc

'--------------------------------------------------------------------------------
' The main program starts here
'
Main:
    PLL_Setup(43, 2, 2, $0300)                  ' Configure the Oscillator to operate the device at 79.23MHz

    Global_wInIndex  = 0                        ' Reset the buffer index
    Clear Global_wInputBuf                      ' Clear the Global_wInputBuf array
'
' Initialise the Filter parameters with the buffer and coefficient details
'
    FIR_StructInit(FIRStruct, cCOEFF, cFILTER_TAPS, Global_wInputBuf, cBUFFER_SIZE)

    ADC_Start()                                 ' Initialise and start the ADC interrupt
    Timer1_Start()                              ' Initialise and start the Timer1 interrupt
    DAC1_Init(36, cDAC_Signed)                  ' Initialise the DAC for signed output
    DAC1L_Enable(0)                             ' Enable the DAC, no interrupts

    While                                       ' Create an infinite loop
        DAC1L_Write(Global_wFilterValue)        ' Write the filter value to the DAC
    Wend                                        ' Do it forever

'---------------------------------------------------------------------------------------
' Configure for internal 7.37MHz oscillator with PLL
' OSC pins are general purpose I/O
'
    Config FBS = BWRP_WRPROTECT_OFF
    Config FGS = GWRP_OFF
    Config FOSCSEL = FNOSC_FRCPLL, IESO_OFF
    Config FOSC = POSCMD_NONE, OSCIOFNC_ON, IOL1WAY_OFF, FCKSM_CSECME
    Config FWDT = WDTPOST_PS256, WINDIS_OFF, FWDTEN_OFF
    Config FPOR = FPWRT_PWR1, ALTI2C_OFF
    Config FICD = ICS_PGD1, JTAGEN_OFF

